# UNBALANCE
INI_WINDOWSIZE = 0
END_THRESHOLD = float(1.2)
SECONDS = int(60)
ANGLE_IN_DEGREE = int(360)
IDEAL_DEGREE = int(90)
DEGREE_RANGE = int(30)
START_FREQ_LIMIT = 0.8
END_FREQ_LIMIT  = 1.2
HARMONIC_ALLOWANCE = 0.2
MAX_AMP_LIMIT_1X = 0.4
START = 2
START_RANGE = 0.0
END = 1.0
HARMONIC_END = 11
HARMONICS_RANGE = 0.5
HARM_END_RNG = 9
# MISALIIGNMENT AND LOOSENESS
FREQUENCY_LIMIT = float(0.20)
RMS_2X_THRESHOLD = float(0.40)
RMS_3X_THRESHOLD = float(0.30)
# BEARING AND GEAR
rmsthreshold = 0.08
ONE = 1
TWO = 2
THREE = 3
four = 4
limit=1.5
FIFTEN=0.15
THIRTY_PERCENT=0.3
THIRTY_FIVE_PERCENT=0.35
FORTY_PERCENT=0.40
FIFTY_PERCENT=0.5
ONE_FIFTY_PERCENT=1.5
ONE_TWENTY_PERCENT=1.2
# RPM 
FREQ_IN_HERTZ = 60 
START_THRESHOLD = float(0.30)
# FFT AVERAGING
RMS_THRESHOLD_RANGE = 3
##data
